package com.kda.data.utils;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.kda.utils.Constantes;

/**
 * Clase encargada de gestionar y simplificar las transacciones y los accesos a
 * la baes de datos.
 * 
 */
public class PersistenceUtil {

	private static volatile boolean inicializado = false;
	private static Boolean bloqueado = new Boolean(true);
	private static EntityManagerFactory entityManagerFactory = null;
	protected Logger logger = LoggerFactory.getLogger(getClass());
	private EntityManager entityManager;

	/**
	 * Inicialización de las transacciones
	 */
	public PersistenceUtil() {
		this(false);
	}

	/**
	 * Inicializaciones de la EntityManagerFactory
	 */
	public PersistenceUtil(boolean soloLectura) {
		inicializar();
		crearEntityManager(soloLectura);
	}

	/**
	 * Creación del EntityManager para la gestión de entidades
	 * 
	 * @param soloLectura
	 */
	public void crearEntityManager(boolean soloLectura) {
		if (entityManager != null) {
			return;
		}
		entityManager = entityManagerFactory.createEntityManager();
		if (soloLectura == false) {
			entityManager.getTransaction().begin();
		}
	}

	/**
	 * Obtenemos el EntityManager
	 */
	public EntityManager getEntityManager() {
		if (entityManager == null) {
			throw new IllegalStateException("No transaction was active!");
		}
		return entityManager;
	}

	public void iniciaTransaccion() {
		if (entityManager == null) {
			return;
		}
		if (entityManager.getTransaction().isActive()) {
			return;
		}

		entityManager.getTransaction().begin();
	}

	/**
	 * Finaliza la transacción actual mediante Rollback o Commit.
	 */
	public void finalizaTransaccion() {
		if (entityManager == null) {
			return;
		}

		try {
			if (entityManager.getTransaction().isActive()) {
				if (entityManager.getTransaction().getRollbackOnly()) {
					entityManager.getTransaction().rollback();
				} else {
					entityManager.getTransaction().commit();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			entityManager.close();
			entityManager = null;
		}
	}

	/**
	 * Marcamos la transacción como de sólo rollback
	 */
	public void marcarRollback() {
		if (entityManager != null) {
			entityManager.getTransaction().setRollbackOnly();
		}
	}

	/**
	 * Se recupera si la transacción es de sólo Rollback
	 * 
	 * @return
	 */
	public boolean isRollbackOnly() {
		return entityManager != null
				&& entityManager.getTransaction().getRollbackOnly();
	}

	/**
	 * Inicialización de la EntityManagerFactory
	 */
	private void inicializar() {
		if (inicializado) {
			return;
		}
		synchronized (bloqueado) {
			if (inicializado) {
				return;
			}
			inicializado = true;

			try {
				entityManagerFactory = Persistence
						.createEntityManagerFactory(Constantes.ENTITY_MANAGER_FACTORY);
			} catch (Throwable t) {
				logger.error(
						"[PersistenceUtil.inicializar] Error al recuperar la entityManagerFactory.",
						t);
				return;
			}
		}
	}

	/**
	 * Ejecuta la consulta pasada por parámetro
	 * 
	 * @param query
	 * @return
	 */
	public QrySimple consulta(String query) {
		if (entityManager == null) {
			throw new IllegalStateException(
					"[PersistenceUtil.consulta] No se ha inicializado el EntityManager.");
		}
		return new QrySimple(entityManager.createQuery(query));
	}

	/**
	 * Guardamos el objeto en la base de datos y retorna el Id generado
	 * 
	 * @param objeto
	 * @return
	 */
	public <T extends IdEntidadInterface> T guardar(T objeto) {
		if (entityManager == null) {
			throw new IllegalStateException(
					"[PersistenceUtil.guardar] No se ha inicializado el EntityManager.");
		}
		if (objeto.getId() > 0L) {
			return entityManager.merge(objeto);
		} else {
			entityManager.persist(objeto);
			return objeto;
		}
	}

	/**
	 * Recupera un objeto mediante su Id
	 * 
	 * @param clase
	 * @param id
	 * @return
	 */
	public <T> T getById(Class<T> clase, long id) {
		if (entityManager == null) {
			throw new IllegalStateException(
					"[PersistenceUtil.guardar] No se ha inicializado el EntityManager.");
		}

		return entityManager.find(clase, id);
	}

	/**
	 * Refrescamos un objeto a partir del de la base de datos
	 * 
	 * @param objeto
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public <T extends IdEntidadInterface> T refresh(T objeto) {
		if (entityManager == null) {
			throw new IllegalStateException(
					"[PersistenceUtil.guardar] No se ha inicializado el EntityManager.");
		}

		return (T) entityManager.find(objeto.getClass(), objeto.getId());
	}
}
